import { Action, ActionCreator, AnyAction } from "typescript-fsa";
export interface ReducerBuilder<InS extends OutS, OutS> {
    case<P>(actionCreator: ActionCreator<P>, handler: Handler<InS, OutS, P>): ReducerBuilder<InS, OutS>;
    caseWithAction<P>(actionCreator: ActionCreator<P>, handler: Handler<InS, OutS, Action<P>>): ReducerBuilder<InS, OutS>;
    cases<P1>(actionCreators: [ActionCreator<P1>], handler: Handler<InS, OutS, P1>): ReducerBuilder<InS, OutS>;
    cases<P1, P2>(actionCreators: [ActionCreator<P1>, ActionCreator<P2>], handler: Handler<InS, OutS, P1 | P2>): ReducerBuilder<InS, OutS>;
    cases<P1, P2, P3>(actionCreators: [ActionCreator<P1>, ActionCreator<P2>, ActionCreator<P3>], handler: Handler<InS, OutS, P1 | P2 | P3>): ReducerBuilder<InS, OutS>;
    cases<P1, P2, P3, P4>(actionCreators: [ActionCreator<P1>, ActionCreator<P2>, ActionCreator<P3>, ActionCreator<P4>], handler: Handler<InS, OutS, P1 | P2 | P3 | P4>): ReducerBuilder<InS, OutS>;
    cases<P>(actionCreators: Array<ActionCreator<P>>, handler: Handler<InS, OutS, P>): ReducerBuilder<InS, OutS>;
    casesWithAction<P1>(actionCreators: [ActionCreator<P1>], handler: Handler<InS, OutS, Action<P1>>): ReducerBuilder<InS, OutS>;
    casesWithAction<P1, P2>(actionCreators: [ActionCreator<P1>, ActionCreator<P2>], handler: Handler<InS, OutS, Action<P1 | P2>>): ReducerBuilder<InS, OutS>;
    casesWithAction<P1, P2, P3>(actionCreators: [ActionCreator<P1>, ActionCreator<P2>, ActionCreator<P3>], handler: Handler<InS, OutS, Action<P1 | P2 | P3>>): ReducerBuilder<InS, OutS>;
    casesWithAction<P1, P2, P3, P4>(actionCreators: [ActionCreator<P1>, ActionCreator<P2>, ActionCreator<P3>, ActionCreator<P4>], handler: Handler<InS, OutS, Action<P1 | P2 | P3 | P4>>): ReducerBuilder<InS, OutS>;
    casesWithAction<P>(actionCreators: Array<ActionCreator<P>>, handler: Handler<InS, OutS, Action<P>>): ReducerBuilder<InS, OutS>;
    build(): (state: InS | undefined, action: {
        type: any;
    }) => OutS;
    (state: InS | undefined, action: AnyAction): OutS;
}
export declare type Handler<InS extends OutS, OutS, P> = (state: InS, payload: P) => OutS;
export declare function reducerWithInitialState<S>(initialState: S): ReducerBuilder<S, S>;
export declare function reducerWithoutInitialState<S>(): ReducerBuilder<S, S>;
export declare function upcastingReducer<InS extends OutS, OutS>(): ReducerBuilder<InS, OutS>;
